"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createResizePlugin = createResizePlugin;
exports.useResizePlugin = useResizePlugin;
var _react = require("react");
var _reactNativeVisionCamera = require("react-native-vision-camera");
/**
 * An instance of the resize plugin.
 *
 * All temporary memory buffers allocated by the resize plugin
 * will be deleted once this value goes out of scope.
 */

/**
 * Get a new instance of the resize plugin.
 *
 * All temporary memory buffers allocated by the resize plugin
 * will be deleted once the returned value goes out of scope.
 */
function createResizePlugin() {
  const resizePlugin = _reactNativeVisionCamera.VisionCameraProxy.initFrameProcessorPlugin('resize');
  if (resizePlugin == null) {
    throw new Error('Cannot find vision-camera-resize-plugin! Did you install the native dependency properly?');
  }
  return {
    resize: (frame, options) => {
      'worklet';

      // @ts-expect-error
      const arrayBuffer = resizePlugin.call(frame, options);
      switch (options.dataType) {
        case 'uint8':
          // @ts-expect-error
          return new Uint8Array(arrayBuffer);
        case 'float32':
          // @ts-expect-error
          return new Float32Array(arrayBuffer);
        default:
          throw new Error(`Invalid data type (${options.dataType})!`);
      }
    }
  };
}

/**
 * Use an instance of the resize plugin.
 *
 * All temporary memory buffers allocated by the resize plugin
 * will be deleted once the component that uses `useResizePlugin()` unmounts.
 */
function useResizePlugin() {
  const plugin = (0, _react.useRef)();
  if (plugin.current == null) plugin.current = createResizePlugin();
  return plugin.current;
}
//# sourceMappingURL=index.js.map